# coding: utf-8
'''
Created on 28 июля 2015 г.

@author: v.popov
'''
import re
from nci._nci_orm import personCursor, person_edu_organizationCursor
from security._security_orm import subjectsCursor, loginsCursor
from common.numbersseries.getNextNo import getNextNoOfSeries
import hashlib
from security.functions import id_generator
from ru.curs.celesta.syscursors import UserRolesCursor
from webservices.functions import uploadCursor
import itertools
import urllib
from ru.curs.celesta.showcase.utils import XMLJSONConverter
import json


def checkSnilsUnique(context, snils, person_id=None):
    u'''Проверяем СНИЛС на уникальность в реестре ФЛ. '''

    person = personCursor(context)
    person.setRange('snils', snils)
    if person_id:
        # проверка на наличие других ФЛ
        person.setFilter('uid', "!'%s'" % person_id)
    if person.tryFindSet():
        return False
    else:
        return True

def getPersonIdBySid(context, sid):
    u'''Получаем ID ФЛ по sid. '''

    subjects = subjectsCursor(context)
    subjects.get(sid)

    return subjects.employeeId

def getPersonRolesEduOrg(context, person_id):
    u'''Получаем роли ФЛ в образовательных организациях. '''

    roles = {'staff': [], 'supervised_student': []}
    peo = person_edu_organizationCursor(context)
    peo.setRange('person_id', person_id)
    for peo in peo.iterate():
        if peo.role in roles:
            if peo.edu_organization_id not in roles[peo.role]:
                roles[peo.role].append(peo.edu_organization_id)
        else:
            roles[peo.role] = [peo.edu_organization_id]

    return roles


def normalizeSnils(snils):
    u'''Приводим СНИЛС к одному виду. '''

    _snils = re.sub(r'\D', '', snils, flags=re.U)
    if len(_snils) == 11:
        # приводим к виду ###-###-### ##
        return "{0}-{1}-{2} {3}".format(_snils[0:3], _snils[3:6], _snils[6:9], _snils[9:])
    else:
        raise Exception(u'СНИЛС должен состоять из 9 значащих и 2 контрольных цифр.')


def snilsNormalizationTriggerWithoutCheck(rec):
    u'''Триггер для приведения СНИЛС к единой форме.'''
    if rec.snils:
        snils = normalizeSnils(rec.snils)
        # приводим к виду ###-###-### ##
        rec.snils = snils


def snilsNormalizationTrigger(rec):
    u'''Триггер для приведения СНИЛС к единой форме.'''
    if rec.snils:
        snils = normalizeSnils(rec.snils)
        context = rec.callContext()
        # приводим к виду ###-###-### ##
        rec.snils = snils
        # проверяем уникальность СНИЛС
        if not checkSnilsUnique(context, rec.snils, rec.uid):
            context.error(u'Запись о физическом лице с данным СНИЛС уже присуствует в реестре.')


def createAccount(context, person_id, password=None, login=None, roles=[]):
    u'''Создаем аккаунт для данного ФЛ. '''

    # получаем общие настройки
    # from nci import generalSettings
    # _generalSettings = generalSettings.getSettingsJSON()

    subjects = subjectsCursor(context)
    logins = loginsCursor(context)
    person = personCursor(context)
    UserRoles = UserRolesCursor(context)
    person.get(person_id)
    # добавляем новую запись в Subjects
    subjects.sid = u'%s%s' % (getNextNoOfSeries(context, 'subjects'), id_generator())
    # subject_id = subjects.sid  # нужен в сервисе
    subjects.name = person.fullname
    subjects.employeeId = person_id
    subjects.insert()
    # создаем аккаунт
    logins.subjectId = subjects.sid
    if not login:
        login = person.snils.replace("-", "").replace(" ", "")  # СНИЛС в качестве логина без - и пробелов
    logins.userName = login
    # генерируем пароль если не указан
    if not password:
        _password = id_generator()
    else:
        _password = password
    pass_hash = hashlib.sha1()
    pass_hash.update(_password)
    logins.password = pass_hash.hexdigest()
    logins.insert()
    # добавляем роли
    for role in roles:
        UserRoles.roleid = role
        UserRoles.userid = subjects.sid
        UserRoles.insert()
    # отправка сообщения об изменении пароля - нужно реализовать
    #

    #
    #### ПОД ВОПРОСОМ, пока норм не работает ######
    """
    if _generalSettings['configuration'] != 'full':
        # выбираем список веб-сервисов
        UserRoles.setRange('userid', subjects.sid)
        subjects.setRange('sid', subject_id)
        logins.setRange('subjectId', subject_id)
        wsError = []
        msWs = list(itertools.chain.from_iterable(_generalSettings['webServiceSettings'].values()))
        for ws in msWs:
            # отправляем данные в каждую систему
            try:
                uploadCursor(context, subjects, 'security', 'subjects',
                              ws['systemName'], ws['userPin'],  ws['wsUrl'], dbaction='i')
                uploadCursor(context, logins, 'security', 'logins',
                              ws['systemName'], ws['userPin'],  ws['wsUrl'], dbaction='i')
                uploadCursor(context, UserRoles, 'celesta', 'UserRoles',
                              ws['systemName'], ws['userPin'],  ws['wsUrl'], dbaction='i')
            except:
                wsError.append(u'Возникла ошибка при синхронизацией с системой %s' % ws['systemName'])
        if wsError:
            context.message(";".join(wsError))
    """


def changePassword(context, person_id, password=None):
    u'''Смена пароля.'''

    # получаем общие настройки
    # from nci import generalSettings
    # _generalSettings = generalSettings.getSettingsJSON()

    subjects = subjectsCursor(context)
    logins = loginsCursor(context)
    subjects.setRange('employeeId', person_id)
    if subjects.count() == 0:
        context.error(u'У данного ФЛ отсутствует аккаунт!')
    elif subjects.count() == 1:
        subjects.first()
        logins.setRange('subjectId', subjects.sid)
        if logins.tryFirst():
            if password:
                _password = password
            else:
                _password = id_generator()
            pass_hash = hashlib.sha1()
            pass_hash.update(_password)
            logins.password = pass_hash.hexdigest()
            logins.update()
            # отправка сообщения об изменении пароля - нужно реализовать

            #
            #### ПОД ВОПРОСОМ, пока норм не работает ######
            """
            if _generalSettings['configuration'] != 'full':
                # выбираем список веб-сервисов
                msWs = list(itertools.chain.from_iterable(_generalSettings['webServiceSettings'].values()))
                logins.setRange('subjectId', subjects.sid)
                wsError = []
                for ws in msWs:
                    # отправляем данные в каждую систему
                    try:
                        uploadCursor(context, logins, 'security', 'logins',
                                  ws['systemName'], ws['userPin'],  ws['wsUrl'])
                    except:
                        wsError.append(u'Возникла ошибка при синхронизацией с системой %s' % ws['systemName'])
                if wsError:
                    context.message(";".join(wsError)) """
            context.message(u'Пароль успешно изменен.')
        else:
            context.error(u'У данного ФЛ отсутствует аккаунт!')
    else:
        context.error(u'У данного ФЛ существует более 2 аккаунтов!')


def getFRMRPersonData(snils):
    u'''Получаем данные по ФЛ из ФРМР по СНИЛС. '''

    # получаем общие настройки
    from nci import generalSettings
    _generalSettings = generalSettings.getSettingsJSON()

    # проверяем, что СНИЛС соответствует Формату ###-###-###-##
    _snils = re.sub(r'\D', '', snils, flags=re.U)
    if len(_snils) == 11:
        # приводим к виду ###-###-### ##
        _snils = "{0}-{1}-{2}-{3}".format(_snils[0:3], _snils[3:6], _snils[6:9], _snils[9:])
    else:
        raise Exception(u'СНИЛС должен состоять из 9 значащих и 2 контрольных цифр.')

    # отправляем запрос на локальный сервис ФРМР
    try:
        f = urllib.urlopen("%s/GetEmployee/%s" % (_generalSettings['localFRMRService']['url'], _snils))
        res_json = XMLJSONConverter.xmlToJson(f.read())
        emp_json = json.loads(res_json)['s:Envelope']['s:Body']['GetEmployeeResponse']['GetEmployeeResult']
    except Exception, e:
        return e

    return {'Employee': emp_json['Employee']} if 'Employee' in emp_json else None
