# coding: utf-8

import json

from common.sysfunctions import toHexForXml, getGridHeight
from nci._nci_orm import education_levelCursor
from nci.xform.specialitiesDeleteCard import checkSpecialitiesUsage
from ru.curs.celesta.showcase.utils import XMLJSONConverter
from umk._umk_orm import specialityCursor


try:
    from ru.curs.showcase.core.jython import JythonDTO
    from ru.curs.showcase.app.api import MessageType, UserMessage
except:
    from ru.curs.celesta.showcase import JythonDTO

def gridData(context, main, add=None, filterinfo=None, session=None, elementId=None,
             sortColumnList=None, firstrecord=None, pagesize=None):

    specialities = specialityCursor(context)
    education_level = education_levelCursor(context)

    # Заголовки
    _header = {
        'uid': ['~~id', 'uid'],
        'name': [u'Название', 'name'],
        'shortname': [u'Короткое название', 'shortname'],
        'education_level': [u'Уровень образования', 'education_level'],
        'is_actual': [u'Актуальная', 'is_actual'],
        'properties': [u'properties', 'properties']
    }
    for column in _header:
        _header[column].append(toHexForXml(_header[column][0]))

    specialities.orderBy('name')
    specialities.limit(firstrecord - 1, pagesize)

    grid_data = {
        'records': {
            'rec':[]
        }
    }
    for row in specialities.iterate():
        currentrow = {}
        currentrow[_header['uid'][2]] = row.uid
        currentrow[_header['name'][2]] = row.name
        currentrow[_header['shortname'][2]] = row.shortname
        if specialities.education_level:
            education_level.get(specialities.education_level)
            currentrow[_header['education_level'][2]] = education_level.name
        else:
            currentrow[_header['education_level'][2]] = ''
        currentrow[_header['is_actual'][2]] = '%s.png' % str(row.is_actual).lower()
        currentrow[_header['properties'][2]] = {
            'event': {
                '@name': 'row_single_click',
                'action': {
                    '#sorted': [
                        {
                            'main_context': 'current'
                        },
                        {
                            'datapanel': {
                                '@type': 'current',
                                '@tab': 'current'
                            }
                        }
                    ]
                }
            }
        }
        grid_data['records']['rec'].append(currentrow)

    return JythonDTO(XMLJSONConverter.jsonToXml(json.dumps(grid_data)),
                     None)

def gridMeta(context, main=None, add=None, filterinfo=None, session=None, elementId=None,
             sortColumnList=None):

    specialities = specialityCursor(context)

    # Заголовки
    _header = {
        'uid': ['~~id', 'uid'],
        'name': [u'Название', 'name'],
        'shortname': [u'Короткое название', 'shortname'],
        'education_level': [u'Уровень образования', 'education_level'],
        'is_actual': [u'Актуальная', 'is_actual'],
        'properties': [u'properties', 'properties']
    }
    for column in _header:
        _header[column].append(toHexForXml(_header[column][0]))

    grid_settings = {}
    grid_settings['gridsettings'] = {
        'labels': {
            'header': 'Специальности'
        },
        'columns': {
            'col': []
        },
        'properties': {
            '@pagesize': 50,
            '@gridWidth': '100%',
            '@gridHeight': getGridHeight(session, numberOfGrids=1, gridHeaderHeight=55, delta=59),
            '@totalCount': specialities.count(),
            '@profile': 'default.properties'
        },
    }
    # Добавляем поля для отображения в gridsettingss
    grid_settings['gridsettings']['columns']['col'].append({'@id': _header['name'][0]})
    grid_settings['gridsettings']['columns']['col'].append({'@id': _header['shortname'][0],
                                                            '@width': '50px'})
    grid_settings['gridsettings']['columns']['col'].append({'@id': _header['education_level'][0],
                                                            '@width': '50px'})
    grid_settings['gridsettings']['columns']['col'].append({'@id': _header['is_actual'][0],
                                                            '@type': 'IMAGE',
                                                            '@width': '50px'})

    return JythonDTO(None,
                     XMLJSONConverter.jsonToXml(json.dumps(grid_settings)))

def gridToolBar(context, main=None, add=None, filterinfo=None, session=None, elementId=None):

    session_json = json.loads(session)['sessioncontext']
    grid_context = session_json['related']['gridContext']
    string_bool = lambda x: x and 'true' or 'false'

    add_style = string_bool(False)
    edit_style = (string_bool(False) if 'currentRecordId' in grid_context
                  else string_bool(True))
    delete_style = string_bool(checkSpecialitiesUsage(context, grid_context['currentRecordId'])
                               if  'currentRecordId' in grid_context else True)

    grid_toolbar = {
        'gridtoolbar': {
            '#sorted': [
                {
                    'item': {
                        '@text': u'Добавить',
                        '@hint': u'Добавить специальность',
                        '@disable': add_style,
                        'action': {
                            '@show_in': 'MODAL_WINDOW',
                            '#sorted': [
                                {
                                    'main_context': 'current'
                                },
                                {
                                    'modalwindow': {
                                        '@caption': u'Добавить специальность',
                                        '@height': '450',
                                        '@width': '500'
                                    }
                                },
                                {
                                    'datapanel': {
                                        '@type': 'current',
                                        '@tab': 'current',
                                        'element': {
                                            '@id': 'specialitiesCard',
                                            'add_context': 'add'
                                        }
                                    }
                                }
                            ]
                        }
                    }
                },
                {
                    'item': {
                        '@text': u'Редактировать',
                        '@hint': u'Редактировать специальность',
                        '@disable': edit_style,
                        'action': {
                            '@show_in': 'MODAL_WINDOW',
                            '#sorted': [
                                {
                                    'main_context': 'current'
                                },
                                {
                                    'modalwindow': {
                                        '@caption': u'Редактировать специальность',
                                        '@height': '450',
                                        '@width': '500'
                                    }
                                },
                                {
                                    'datapanel': {
                                        '@type': 'current',
                                        '@tab': 'current',
                                        'element': {
                                            '@id': 'specialitiesCard',
                                            'add_context': 'edit'
                                        }
                                    }
                                }
                            ]
                        }
                    }
                },
                {
                    'item': {
                        '@text': u'Удалить',
                        '@hint': u'Удалить специальность',
                        '@disable': delete_style,
                        'action': {
                            '@show_in': 'MODAL_WINDOW',
                            '#sorted': [
                                {
                                    'main_context': 'current'
                                },
                                {
                                    'modalwindow': {
                                        '@caption': 'Удалить специальность',
                                        '@height': '140',
                                        '@width': '400'
                                    }
                                },
                                {
                                    'datapanel': {
                                        '@type': 'current',
                                        '@tab': 'current',
                                        'element': {
                                            '@id': 'specialitiesDeleteCard',
                                            'add_context': ''
                                        }
                                    }
                                }
                            ]
                        }
                    }
                }
            ]
        }
    }

    return XMLJSONConverter.jsonToXml(json.dumps(grid_toolbar))
