# coding: utf-8
'''
Created on 04.08.2014
Карточка ФЛ
@author: v.popov
'''
from java.text import SimpleDateFormat
import json
import re

from com.jayway.jsonpath import JsonPath
from common.numbersseries.getNextNo import getNextNoOfSeries
from nci._nci_orm import personCursor, edu_organizationCursor, \
    person_edu_organizationCursor
from nci.functions.persons import checkSnilsUnique
from ru.curs.celesta.showcase.utils import XMLJSONConverter
from security.functions import userHasPermission

try:
    from ru.curs.showcase.core.jython import JythonDTO
    from ru.curs.showcase.app.api import MessageType, UserMessage
except:
    from ru.curs.celesta.showcase import JythonDTO


def cardData(context, main=None, add=None, filterinfo=None, session=None, elementId=None):
    u'''Карточка ФЛ'''
    sid = JsonPath.read(session, "$.sessioncontext.sid")
    person = personCursor(context)
    if add == 'edit':
        person_id = JsonPath.read(session, "$.sessioncontext.related.gridContext.currentRecordId")
        person.get(person_id)
    else:
        person_id = ""

    xformsdata = {"schema":
                      {"@xmlns":"",
                        "record":
                            {"uid": person_id
                           , "surname": person.surname
                           , "name": person.name
                           , "birthDate": SimpleDateFormat("yyyy-MM-dd").format(person.birthDate) if person.birthDate else ""
                           , "patronymic": person.patronymic
                           , "email": person.email
                           , "phone": person.phone
                           , "phoneExtra": person.phoneExtra
                           , "snils": person.snils
                           , "sex": person.sex
                           , "personal_data_agree": "true" if person.personal_data_agree or person.personal_data_agree is None else "false"
                           , "readonly": "false"
                            }
                        }
                      }

    xformssettings = {"properties":{"event":[{"@name":"single_click",
                                              "@linkId": "1",
                                              "action":{"@keep_user_settings": "true",
                                                        '#sorted': [
                                                            {"main_context": "current"},
                                                            {"datapanel": {"@type": "current",
                                                                      "@tab": "current",
                                                                      "element": {"@id":"personGrid",
                                                                                  "add_context": ""
                                                                                  }
                                                                      }}]
                                                        }
                                              }]
                                    }
                      }

    return JythonDTO(XMLJSONConverter.jsonToXml(json.dumps(xformsdata)),
                     XMLJSONConverter.jsonToXml(json.dumps(xformssettings)))

def cardSave(context, main=None, add=None, filterinfo=None, session=None, elementId=None, xformsdata=None):
    u'''Сохранение данных о ФЛ. '''

    person = personCursor(context)

    data_dict = json.loads(xformsdata)["schema"]["record"]

    if not data_dict["uid"] == '':
        # Редактирование уже существующей
        person.get(data_dict["uid"])
    person.surname = data_dict["surname"]
    person.name = data_dict["name"]
    person.patronymic = data_dict["patronymic"]
    person.snils = data_dict["snils"]
    person.sex = data_dict["sex"]
    person.email = data_dict["email"]
    person.phone = data_dict["phone"]
    person.phoneExtra = data_dict["phoneExtra"]
    person.birthDate = SimpleDateFormat("yyyy-MM-dd").parse(data_dict["birthDate"]) if data_dict["birthDate"] else None
    person.personal_data_agree = data_dict["personal_data_agree"] == "true"
    person.fullname = u'%s %s %s' % (person.surname or '',
                                     person.name or '',
                                     person.patronymic or '')
    if data_dict["uid"] == '':
        # Добавление новой записи в person
        person.uid = getNextNoOfSeries(context, 'persons')
        person.insert()
    else:
        # Редактирование уже существующей
        person.update()


def checkSnils(context, main=None, add=None, filterinfo=None, session=None, data=None):
    u'''функция сабмишна для проверки СНИЛС.'''

    j1 = json.loads(data)
    _s1 = j1['schema']['record']['snils']
    if _s1 == '':
        return data
    check_sum = check_snils(_s1)
    if check_sum == 999:
        return UserMessage(u'''Ошибка в заполнении СНИЛС! (должно быть 11 значащих цифр)''', MessageType.ERROR)
    elif check_sum == 'letters':
        return UserMessage(u'''Ошибка в заполнении СНИЛС! Должны быть только цифры, знаки - и пробелы!''', MessageType.ERROR)
    elif check_sum != int(_s1[-2:]):
        return UserMessage(u'''Ошибка в заполнении СНИЛС!
        Неверно указано контрольное число (должно быть %s)''' % check_sum, MessageType.ERROR)

    return data

def check_snils(snils, digits=True):
    
    if re.search(r'[a-zа-я]', snils, flags=re.U|re.I):
        return 'letters'
    
    if len(snils.replace('-', '').replace(' ', '')) <> 11:
        return 999

    def snils_csum(snils):
        k = range(9, 0, -1)
        pairs = zip(k, [int(x) for x in snils.replace('-', '').replace(' ', '')[:-2]])
        return sum([k * v for k, v in pairs])

    csum = snils_csum(snils)

    while csum > 101:
        csum %= 101
    if csum in (100, 101):
        csum = 0

    return csum

def checkEmail(context, main=None, add=None, filterinfo=None, session=None, data=None):
    u'''функция сабмишна для проверки Email.'''

    j1 = json.loads(data)
    mess1 = u''''''
    _email = j1['schema']['record']['email']

    _pr3 = re.search(r'\A\w+[\.a-zA-Z0-9_-]+@[a-zA-Z0-9][\.a-zA-Z0-9_-]+\.[a-zA-Z]{2,4}\Z', _email)

    if not _pr3 and _email != '':
        mess1 = chr(13) + u'Неверно указан адрес электронной почты!'

    if mess1 != '':
        return UserMessage(mess1, MessageType.ERROR)
    return data

def deleteCardData(context, main=None, add=None, filterinfo=None, session=None, elementId=None):
    u'''Данные для карточки.'''

    string_bool = lambda x: x and 'true' or 'false'

    # Структура данных
    xforms_data = {
        'schema': {
            '@xmlns': '',
            'data': {
                'form_text': 'Вы действительно хотите удалить запись от физическом лице?'
            }
        }
    }

    # Первоначальная xforms_settings
    xforms_settings = {
        'properties': {
            'event': [
                {
                    '@name': 'single_click',
                    '@linkId': '1',
                    'action': {
                        '@keep_user_settings': string_bool(True),
                        '#sorted': [
                          {'main_context': 'current'},
                          {'datapanel': {
                              '@type': 'current',
                              '@tab': 'current',
                              'element': {
                                  '@id': 'personGrid',
                                  'add_context': ''
                              }
                          }}]
                    }
                }
            ]
        }
    }

    return JythonDTO(XMLJSONConverter.jsonToXml(json.dumps(xforms_data)),
                     XMLJSONConverter.jsonToXml(json.dumps(xforms_settings)))


def deleteCardSave(context, main=None, add=None, filterinfo=None, session=None, elementId=None, xformsdata=None):
    u'''Сохранение карточки. '''

    person = personCursor(context)
    current_id = json.loads(session)['sessioncontext']['related']['gridContext']['currentRecordId']
    person.get(current_id)
    person.delete()